!=========================================================================
! Copyright (C) GemTalk Systems 1986-2016.  All Rights Reserved.
!
! $Id: segment.gs,v 1.7 2008-01-09 19:02:36 stever Exp $
!
! Superclass Hierarchy:
!   Segment, Object.
!
!=========================================================================

removeallmethods Segment
removeallclassmethods Segment

category: 'For Documentation Installation only'
classmethod: Segment
installDocumentation

| doc txt |
doc := GsClassDocumentation newForClass: self.

txt := (GsDocText new) details:
'Each Repository is composed of an integral number of Segments.  A Segment has
 the following properties:

 * Ownership.  This is the smallest unit of ownership for accounting and
   authorization purposes.  Each Segment is owned by one and only one user.

 * Authorization unit.  A user may control access to objects by placing them in
   a Segment with a known authorization.  Prior to reading or writing an object
   in a Segment, users must be authorized to perform the desired action.
   However, during a transaction, once a user is authorized to read one object
   in a Segment, that user can read any object in the same Segment.

 Segments and Repositories are discussed in more detail in the GemStone
 Programming Guide.'.
doc documentClassWith: txt.

txt := (GsDocText new) details:
'This variable is obsolete in GemStone v5.0 and is provided only for
 compatibility with earlier releases.' .
doc documentClassVar: #AuthorizationMasks with: txt.

txt := (GsDocText new) details:
'This variable is obsolete in GemStone v5.0 and is provided only for
 compatibility with earlier releases.' .
doc documentClassVar: #AuthorizationFactors with: txt.

txt := (GsDocText new) details:
'This variable is obsolete in GemStone v4.1 and is provided only for
 compatibility with earlier releases.' .
doc documentClassVar: #AuthorizationSymbols with: txt.

txt := (GsDocText new) details:
'The Repository containing the Segment.'.
doc documentInstVar: #itsRepository with: txt.

txt := (GsDocText new) details:
'A UserProfile indicating the owner of the Segment.'.
doc documentInstVar: #itsOwner with: txt.

txt := (GsDocText new) details:
'An IdentitySet of canonical Strings.  Each String must be an element of
 AllGroups, and represents a group of users who are authorized to access
 the Segment for reading.'.
doc documentInstVar: #groupsRead with: txt.

txt := (GsDocText new) details:
'An IdentitySet of canonical Strings.  Each String must be an element of
 AllGroups, and represents a group of users who are authorized to access
 the Segment for writing.'.
doc documentInstVar: #groupsWrite with: txt.

txt := (GsDocText new) details:
'A SmallInteger specifying authorization for the owner to access the Segment.
 0 = no access, 1 = read access, 2 = write access.'.
doc documentInstVar: #ownerAuthorization with: txt.

txt := (GsDocText new) details:
'A SmallInteger specifying world authorization to access the Segment.
 0 = no access, 1 = read access, 2 = write access.'.
doc documentInstVar: #worldAuthorization with: txt.

txt := (GsDocText new) details:
'Reserved for future use.'.
doc documentInstVar: #spare1 with: txt.

txt := (GsDocText new) details:
'Methods in this category are obsolete and are provided only for compatibility
 with earlier releases of GemStone.  They will be removed in a future release.'.
doc documentCategory: #'Backward Compatibility' with: txt.

txt := (GsDocText new) details:
'In general, if you are not the data curator, you may only change the
 authorization for a Segment if you are the owner of that Segment.  To execute
 an authorization message on a Segment you do not own, you must have the
 SegmentProtection privilege.

 Exercise caution when changing the authorization for any Segment that a user
 may be using as his or her default or current Segment (whether or not the user
 owns the affected Segment).  If a user attempts to commit a transaction, but
 has created objects in a Segment for which he or she no longer has write
 authorization, an error will be generated.  In addition, if a user is no
 longer authorized to write in his or her default Segment, the user''s GemStone
 session will be terminated and the user will be unable to log back in to
 GemStone.'.
doc documentCategory: #'Updating Authorization' with: txt.

self description: doc.
%

category: 'Instance Creation'
classmethod: Segment
new

"Disallowed.  To create a new Segment, use newInRepository: instead."

self shouldNotImplement: #new
%

category: 'Instance Creation'
classmethod: Segment
newInRepository: aRepository

"Returns a new Segment in the Repository aRepository.  If the maximum number of
 Segments has already been created for aRepository, this generates an error.
 The new Segment has the default authorization of W----- (owner can read and
 write).

 Requires the SegmentCreation privilege."

<primitive: 291>
aRepository _validateClass: Repository.
self _primitiveFailed: #newInRepository: .
self _uncontinuableError
%

category: 'Accessing'
method: Segment
groups

"Returns an IdentitySet, the set of user groups (Strings) that are
 explicitly authorized to read or write in this Segment."

| result |
groupsRead ~~ nil ifTrue:[ result := groupsRead ] 
                 ifFalse:[ result := IdentitySet new ].
groupsWrite ~~ nil ifTrue:[ result := result + groupsWrite ].
^ result
%

category: 'Accessing'
method: Segment
number

"Returns the index of the receiver in its Repository.  Return
 -1 if the Repository does not contain the receiver."

| index |

(index := itsRepository indexOf: self) == 0
  ifTrue: [ ^ -1 ].
^index
%

category: 'Accessing'
method: Segment
owner

"Returns the UserProfile of the receiver's owner."

^ itsOwner
%

category: 'Accessing'
method: Segment
repository

"Returns the Repository containing the receiver."

^ itsRepository
%


category: 'Updating'
method: Segment
owner: aUserProfile

"Redefines the receiver's owner to be the user represented by aUserProfile.
 To execute this method, you must be authorized to write in the receiver's
 Segment (customarily, the DataCurator Segment)."

itsOwner := aUserProfile
%

category: 'Formatting'
method: Segment
asString

"Returns a formatted String that contains the following information:

 The name of the Repository containing the receiver and the index of
 the receiver in that Repository.

 * The user ID of the receiver's owner.

 * The Symbol that defines each user group which is authorized to read or
   write in the receiver.

 * Information about whether the receiver's owner, each group, or all
   users are authorized to read or write in the receiver.

 For example, the message 'System myUserProfile defaultSegment asString'
 returns a String that resembles the following:

 aSegment, Number 1 in Repository SystemRepository
 Owner SystemUser write
 World read

 If the receiver is not a Segment in the SystemRepository, the String contains
 'NOT IN REPOSITORY' in place of the index."

| lf index tempString theNumberOfTheSegment theGroupAuthString |

lf := Character lf.
tempString := String new.  " string to return "
theGroupAuthString := String new.  " string of group authorizations"
theNumberOfTheSegment := String new.

"Get the Index in SystemRepository or the error text: NOT IN REPOSITORY"
(index := itsRepository indexOf: self) == 0
  ifFalse: [ theNumberOfTheSegment addAll: 'Number ';
                                   addAll: (index asString) ]
  ifTrue: [ theNumberOfTheSegment addAll: 'NOT IN REPOSITORY' ].

"groups and build a single string to describe group authorizations."
groupsRead size > 0 ifTrue:[
  (groupsRead sortAscending:  '' ) do:
    [:groupName | theGroupAuthString add: lf;
                 addAll: 'Group ';
                 addAll: (groupName asString);
                 add: ' ';
                 addAll: 'read' .
                ].
  ].
groupsWrite size > 0 ifTrue:[
  (groupsWrite sortAscending:  '' ) do:
     [:groupName | theGroupAuthString add: lf;
                 addAll: 'Group ';
                 addAll: (groupName asString);
                 add: ' ';
                 addAll: 'write' .
                ].
  ].

tempString add: lf; " Build the string to return"
             addAll: 'aSegment, ';
             addAll: theNumberOfTheSegment;
             addAll: ' in Repository ';
             addAll: (self repository name);
             add: lf;
             addAll: 'Owner ';
             addAll: (self owner userId);
             add: ' ';
             addAll: (self ownerAuthorization);
             addAll: theGroupAuthString;
             add: lf;
             addAll: 'World ';
             addAll: (self worldAuthorization).

^tempString
%

category: 'Clustering'
method: Segment
clusterDepthFirst

"This method clusters the receiver.  (Overrides the inherited method, which
 also clusters all instance variables.)  Returns true if the receiver has 
 already been clustered during the current transaction, false otherwise."

  self cluster ifTrue:[ ^ true ].
  groupsRead cluster .
  groupsWrite cluster .
^ false
%

category: 'Copying'
method: Segment
copy

"Returns the receiver. Copies of segments are not made."

^ self.
%

category: 'Copying'
method: Segment
_deepCopyWith: copiedObjDict

"Private. Used internally to implement deepCopy."

^ self.
%

category: 'Accessing Authorization'
method: Segment
authorizationForGroup: aGroupString

"Returns a Symbol that defines whether the specified group is authorized to
 write (and read) in this Segment (#write), to read only (#read), or neither
 (#none)."

| canonGroup |
canonGroup := AllGroups at: aGroupString otherwise: nil .
canonGroup == nil ifTrue:[ ^ #none ].

groupsWrite size > 0 ifTrue:[
  (groupsWrite includesIdentical: canonGroup) ifTrue:[ ^ #write ]
  ].
groupsRead size > 0 ifTrue:[ 
  (groupsRead includesIdentical: canonGroup) ifTrue:[ ^ #read ]
  ].
^ #none
%

category: 'Accessing Authorization'
method: Segment
authorizationForUser: aUserProfile

"Returns a Symbol that describes the authorization that the given user has for
 the receiver."

| userGroups curAuth |

"Consider world authorization"
curAuth := worldAuthorization.
curAuth == 2 ifTrue: [ ^ #write ].

"Owner authorization applicable?"
itsOwner == aUserProfile ifTrue: [
  ownerAuthorization == 2 ifTrue:[ ^ #write ].
  curAuth := curAuth bitOr: ownerAuthorization .
  ].

"Consider group authorizations..."
userGroups := aUserProfile groups .
userGroups size > 0 ifTrue:[
  groupsWrite size > 0 ifTrue:[
    (groupsWrite * userGroups) size > 0 ifTrue:[ ^ #write ].
    ].
  groupsRead size > 0 ifTrue:[
    (groupsRead * userGroups) size > 0 ifTrue:[ curAuth := curAuth bitOr: 1 ].
    ].
  ].
   
curAuth == 1 ifTrue:[ ^ #read ].
^#none
%

category: 'Accessing Authorization'
method: Segment
ownerAuthorization

"Returns a Symbol that defines whether the Segment's owner is authorized to
 write (and read) in this Segment (#write), to read only (#read), or neither
 (#none)."

^ #( #none #read #write) at: ownerAuthorization + 1
%

category: 'Accessing Authorization'
method: Segment
worldAuthorization

"Returns a Symbol that defines whether all system users are authorized to write
 (and read) in this Segment (#write), to read only (#read), or neither
 (#none)."

^ #( #none #read #write) at: worldAuthorization + 1
%

category: 'Accessing Authorization'
method: Segment
groupsWithAuthorization: anAccessSymbol

"Returns an IdentitySet of group Strings of all groups with the authorization of
 anAccessSymbol (one of #write, #read, or #none) for the receiver."

| result |

( #( #write #read #none) includesValue: anAccessSymbol)
  ifFalse: [  ^ self _error: #segErrBadAuthKind args: #[anAccessSymbol] ].

result := IdentitySet new .
AllGroups valuesDo:[ :aGroup |
  (self authorizationForGroup: aGroup) == anAccessSymbol ifTrue:[
    result add: aGroup
    ].
  ].
^ result
%

category: 'Accessing Authorization'
method: Segment
usersWithAuthorization: anAccessSymbol

"Returns a UserProfileSet containing all users with the authorization of
 anAccessSymbol (one of #write, #read, or #none) for the receiver."

| result |

( #( #write #read #none) includesValue: anAccessSymbol)
  ifFalse: [  ^ self _error: #segErrBadAuthKind args: #[anAccessSymbol] ].
   
result := UserProfileSet new .
AllUsers do:[:aUserProfile |
  (self authorizationForUser: aUserProfile) == anAccessSymbol ifTrue:[
     result add: aUserProfile
     ].
  ].
^ result
%

category: 'Updating Authorization'
method: Segment
ownerAuthorization: anAuthorizationSymbol

"Redefines the authorization for the Segment's owner to one of the following
 authorization Symbols:

 * #write (the Segment's owner can both read and write in this Segment).

 * #read (read only).

 * #none (neither read nor write permission).

 Generates an error if anAuthorizationSymbol is not one of (#read, #write,
 #none).

 Requires the SegmentProtection privilege, if the segment is not owned
 by the GemStone UserProfile under which this session is running."

 | authInt |
 authInt := #( #none #read #write ) indexOfValue: anAuthorizationSymbol .
 authInt == 0 ifTrue:[ 
   ^ self _error: #segErrBadAuthKind args: #[anAuthorizationSymbol] 
   ].
 self _authAt: 5 put: (authInt - 1) 
%

category: 'Updating Authorization'
method: Segment
worldAuthorization: anAuthorizationSymbol

"Redefines the authorization for all users to one of the following
 authorization Symbols:

 * #write (all users can both read and write in this Segment).

 * #read (read only).

 * #none (neither read nor write permission).

 Generates an error if anAuthorizationSymbol is not one of (#read, #write,
 #none).

 Requires the SegmentProtection privilege, if the segment is not owned
 by the GemStone UserProfile under which this session is running."

 | authInt |
 authInt := #( #none #read #write ) indexOfValue: anAuthorizationSymbol .
 authInt == 0 ifTrue:[ 
   ^ self _error: #segErrBadAuthKind args: #[anAuthorizationSymbol] 
   ].
 self _authAt: 6 put: (authInt - 1) 
%

! fix 9745
category: 'Updating'
method: Segment
_authorizationChanged

"Notify the virtual machine that state related to some segment has
 been changed."

self _authAt: 0 put: nil
%

category: 'Private'
method: Segment
_authAt: anOffset put: aValue

"offset   action 
 ------   ---------
  0       value ignored, notify virtual machine that some change to
          authorization occurred.
  5       assign to ownerAuthorization instance variable
  6       assign to worldAuthorization instance variable
  3       add to groupsRead IdentitySet, create set if needed.
  4       add to groupsWrite IdentitySet, create set if needed.
 -3       remove from groupsRead IdentitySet
 -4       remove from groupsWrite IdentitySet
  7       assign new value to groupsRead (for repository conversion)
  8       assign new value to groupsWrite (for repository conversion)"

<primitive: 292>
self _primitiveFailed: #_authAt:put: .
self _uncontinuableError
%

category: 'Execution'
method: Segment
setCurrentWhile: aBlock

"Sets the receiver to be the current segment while the given block
 executes.  Catches all errors and reinstalls the current segment
 to avoid having the receiver be left as the current segment.  Returns
 the result of evaluating the zero-argument block.

 Note that the argument, aBlock, should not have any returns in it.  Such
 returns will bypass the restoration of the current segment that is normally
 performed after the block has been evaluated."

| oldseg result |

oldseg := System currentSegment.

"If the receiver is the current segment, just returns the evaluation of
 the block - doesn't need error handlers since the current segment won't
 be changed."
oldseg == self ifTrue: [ ^aBlock value ].

Exception category: nil number: nil
  do: [:ex:cat:num:args |
    (oldseg isKindOf: Segment) ifTrue: [
      System currentSegment: oldseg.
      ].
     " following ifFalse only applies to Geode"
     " ifFalse: [
        GsFile stderr nextPutAll:
            'Unable to restore segment in setCurrentWhile:'
        ].  "
     ex resignal: cat number: num args: args.
     System currentSegment: self.
    ].

System currentSegment: self.
result := aBlock value.
System currentSegment: oldseg.
^result
%

category: 'Backward Compatibility'
method: Segment
groupNo: groupIndex group: aGroupString authorization: anAuthorizationSymbol

"Obsolete in GemStone 4.1.  Use the group:authorization: method instead."

^ self group: aGroupString authorization: anAuthorizationSymbol
%

category: 'Updating Authorization'
method: Segment
group: aGroupString authorization: anAuthorizationSymbol

"Redefines the authorization for the specified group to one of the following
 authorization Symbols:

 * #write (members of the group can both read and write in this Segment).
 * #read (read only).
 * #none (neither read nor write permission).

 This method generates an error if aGroupString is not an element of
 the global object AllGroups, or if anAuthorizationSymbol is not one of (#read,
 #write, #none).

 Requires the SegmentProtection privilege, if the segment is not owned
 by the GemStone UserProfile under which this session is running."

| theGroup |

"Test for errInvalidAuthorization "
( #( #write #read #none) includesValue: anAuthorizationSymbol)
ifFalse: [  ^ self _error: #segErrBadAuthKind args: #[anAuthorizationSymbol] ].

"Get canonical group string from AllGroups."
theGroup := AllGroups _validateGroupString: aGroupString .
  
"modify the group collections for this segment as appropriate."
anAuthorizationSymbol = #write ifTrue:[ 
  self _authAt: -3 put: theGroup "readGroups remove: theGroup ifAbsent:[]" .
  self _authAt: 4 put: theGroup  "writeGroups _addSymbol: theGroup " .
  ]
ifFalse:[
  anAuthorizationSymbol = #read ifTrue:[
    self _authAt: -4 put: theGroup "writeGroups remove: theGroup ifAbsent:[]" .
    self _authAt: 3 put: theGroup  "readGroups _addSymbol: theGroup " .
    ]
  ifFalse:[ "none" 
    self _authAt: -3 put: theGroup "readGroups remove: theGroup ifAbsent:[]" .
    self _authAt: -4 put: theGroup "writeGroups remove: theGroup ifAbsent:[]" .
    ].
  ].
%

! nameOfGroupAt: deleted 

category: 'Accessing Authorization'
method: Segment
currentUserCanRead

"Returns true if the current user has read authorization for the
 receiver, false if not."

<primitive: 152>
self _primitiveFailed: #currentUserCanRead .
self _uncontinuableError
%

!  fix bug 7668

category: 'Accessing Authorization'
method: Segment
currentUserCanWrite

"Returns true if the current user has write authorization for the
 receiver, false if not."

<primitive: 153>
self _primitiveFailed: #currentUserCanWrite .
self _uncontinuableError
%

category: 'Storing and Loading'
method: Segment
writeTo: aPassiveObject

"Instances of Segment cannot be converted to passive form.  This method writes
 nil to aPassiveObject and stops GemStone Smalltalk execution with a notifier."

  aPassiveObject writeObject: nil.
  self _error: #rtErrAttemptToPassivateInvalidObject.
%

! _oldAuthorizationFor: deleted
! _migrate40to41  deleted
category: 'Repository Conversion'
method: Segment
_migrateGroups

"Converts the groups collections from SymbolSets to IdentitySets of Strings."

| saveSeg |
saveSeg := System currentSegment .
System currentSegment: DataCuratorSegment.
groupsRead ~~ nil ifTrue:[
  self _authAt:7 put: (AllGroups _migrateSymbolSet: groupsRead ) .
  ].
groupsWrite ~~ nil ifTrue:[
  self _authAt:8 put: (AllGroups _migrateSymbolSet: groupsWrite) . 
  ].
System currentSegment: saveSeg.
%

